#!/usr/bin/python
# -*- coding: utf-8 -*-
import pandas as pd
import numpy as np

TP = lambda pred, label: len(pred&label)
FP = lambda pred, label: len(pred-label)
FN = lambda pred, label: len(label-pred-set(['']))
metric = lambda pred, label: np.array([TP(pred, label), FP(pred, label), FN(pred, label)])
rp = lambda TP, Fx: TP/(TP+Fx)
f1 = lambda r, p: 2*r*p/(r+p)

def cal_rpf(value):
    """计算recall，precision，f1值
    
    Arguments:
        value {np.array} -- [TP, FP, FN]
    
    Returns:
        np.array -- [r, p, f]
    """

    r = rp(value[0], value[2])
    p = rp(value[0], value[1])
    f = f1(r, p)
    return np.array([r, p, f])

def wmean_score(metrics, weight=True):
    """返回对三个预测结果加权之后的recall，precision，f1值
    
    Arguments:
        metrics {dict} -- 原本各个预测的metrics
    
    Keyword Arguments:
        weight {bool} -- 是否对不同预测使用不同权值 (default: {True})
    
    Returns:
        np.array -- [r, p, f]
    """

    metrics = metrics.copy()
    if weight:
        metrics['yf'] *= 0.2  
        metrics['bz'] *= 0.3  
        metrics['zy'] *= 0.5
    overall = np.array([.0] * 3)
    for key, value in metrics.items():
        overall += value
    w_score = cal_rpf(overall)
    return w_score


def read_xlsx(filename):
    """将xlsx文件加载为需要的DataFrame类型
    
    Arguments:
        filename {str} -- 文件路径
    
    Returns:
        pandas.DataFrame -- 需要的DataFrame对象

    Notes:
        会将NaN替换为''
    """

    table = pd.read_excel(filename)
    table.fillna('', inplace=True)
    return table

def plot_evaluation(pred_table, label_table):
    """评价结果
    
    Arguments:
        pred_table {padas.DataFrame} -- 预测的表格结果
        label_table {pandas.DataFrame} -- 标注的表格结果
    
    Raises:
        Exception -- 两个表格长度不一致
    
    Returns:
        pd.DataFrame -- 返回一个评价结果的DataFrame，每行分别为
        
    Notes:
        需要保证传入的DataFrame中，预测不存在的结果用''代替
    """

    mapping = {
        'yf': '肿瘤原发部位',
        'bz': '原发病灶大小',
        'zy': '转移部位'
    }

    metrics = {
        'yf': np.array([.0] * 3),
        'bz': np.array([.0] * 3),
        'zy': np.array([.0] * 3),
    }
    if pred_table.shape != label_table.shape:
        raise Exception("Size not match")
    for (pred_index, pred_row), (label_index, label_row) in zip(pred_table.iterrows(), label_table.iterrows()):


        pred_yf = set(pred_row['肿瘤原发部位'].split(','))
        pred_bz = set(pred_row['原发病灶大小'].split(','))
        pred_zy = set(pred_row['转移部位'].split(','))
        label_yf = set(label_row['肿瘤原发部位'].split(','))
        label_bz = set(label_row['原发病灶大小'].split(','))
        label_zy = set(label_row['转移部位'].split(','))

        metrics['yf'] += metric(pred_yf, label_yf)
        metrics['bz'] += metric(pred_bz, label_bz)
        metrics['zy'] += metric(pred_zy, label_zy)

    table = pd.DataFrame(columns=['recall', 'precision', 'f1'])
    for key, value in metrics.items():
        table.loc[key] = cal_rpf(value)
    table.rename(mapping, axis='index', inplace=True)
    table.loc['overall'] = wmean_score(metrics, weight=False)
    table.loc['overall_weighted'] = wmean_score(metrics)

    print(table.head())
    return table


if __name__ == '__main__':
    pred = read_xlsx('./pred.xlsx')
    label = read_xlsx('./label.xlsx')
    plot_evaluation(pred, label)
